@@ -2177,9 +2177,10 @@ gen7_composite_set_target(struct sna_composite_op *op, PicturePtr dst)
 			return FALSE;
 
 		op->dst.bo = priv->gpu_bo;
-		if (!sna_damage_is_all(&priv->gpu_damage,
-				       op->dst.width, op->dst.height))
-			op->damage = &priv->gpu_damage;
+		op->damage = &priv->gpu_damage;
+		if (sna_damage_is_all(&priv->gpu_damage,
+				      op->dst.width, op->dst.height))
+			op->damage = NULL;
 	}
 
 	get_drawable_deltas(dst->pDrawable, op->dst.pixmap,
