@@ -121,13 +121,21 @@ static const char *_debug_describe_damage(char *buf, int max,
 	if (str_max > sizeof(damage_str))
 		str_max = sizeof(damage_str);
 
-	sprintf(damage_str, "[%d : ...]", damage->n);
-	snprintf(buf, max, "[[(%d, %d), (%d, %d)]:  %s + %s]",
-		 damage->extents.x1, damage->extents.y1,
-		 damage->extents.x2, damage->extents.y2,
-		 _debug_describe_region(region_str, str_max,
-					&damage->region),
-		 damage_str);
+	if (damage->mode == DAMAGE_ALL) {
+		snprintf(buf, max, "[[(%d, %d), (%d, %d)]: all]",
+			 damage->extents.x1, damage->extents.y1,
+			 damage->extents.x2, damage->extents.y2);
+		assert(damage->n == 0);
+	} else {
+		sprintf(damage_str, "[%d : ...]", damage->n);
+		snprintf(buf, max, "[[(%d, %d), (%d, %d)]:  %s %c %s]",
+			 damage->extents.x1, damage->extents.y1,
+			 damage->extents.x2, damage->extents.y2,
+			 _debug_describe_region(region_str, str_max,
+						&damage->region),
+			 damage->mode == DAMAGE_SUBTRACT ? '-' : '+',
+			 damage_str);
+	}
 
 	return buf;
 }
@@ -947,6 +955,11 @@ struct sna_damage *_sna_damage_is_all(struct sna_damage *damage,
 	if (damage->region.data)
 		return damage;
 
+	assert(damage->extents.x1 == 0 &&
+	       damage->extents.y1 == 0 &&
+	       damage->extents.x2 == width &&
+	       damage->extents.y2 == height);
+
 	return _sna_damage_all(damage, width, height);
 }
 
@@ -1125,10 +1138,11 @@ static int _sna_damage_contains_box(struct sna_damage *damage,
 		__sna_damage_reduce(damage);
 
 	ret = pixman_region_contains_rectangle(&damage->region, (BoxPtr)box);
-	if (damage->n == 0)
+	if (ret != PIXMAN_REGION_OUT || damage->n == 0)
 		return ret;
 
-	return ret == PIXMAN_REGION_IN ? PIXMAN_REGION_IN : PIXMAN_REGION_OUT;
+	__sna_damage_reduce(damage);
+	return pixman_region_contains_rectangle(&damage->region, (BoxPtr)box);
 }
 
 #if DEBUG_DAMAGE
@@ -1229,24 +1243,14 @@ static int _sna_damage_get_boxes(struct sna_damage *damage, BoxPtr *boxes)
 	return REGION_NUM_RECTS(&damage->region);
 }
 
-struct sna_damage *_sna_damage_reduce(struct sna_damage *damage,
-				      int width, int height)
+struct sna_damage *_sna_damage_reduce(struct sna_damage *damage)
 {
-	DBG(("%s(width=%d, height=%d)\n", __FUNCTION__, width, height));
-
-	if (damage->n)
-		__sna_damage_reduce(damage);
+	DBG(("%s\n", __FUNCTION__));
 
+	__sna_damage_reduce(damage);
 	if (!pixman_region_not_empty(&damage->region)) {
 		__sna_damage_destroy(damage);
 		damage = NULL;
-	} else {
-		if (damage->region.data == NULL &&
-		    damage->extents.x1 <= 0 &&
-		    damage->extents.y1 <= 0 &&
-		    damage->extents.x2 >= width &&
-		    damage->extents.y2 >= height)
-			damage = _sna_damage_all(damage, width, height);
 	}
 
 	return damage;
